@@ -1,8 +1,9 @@
 import http.client
 import logging
 import os
+import pathlib
 from base64 import b64decode
-from typing import List, TextIO, Type
+from typing import Iterable, List, Type
 
 import click
 
@@ -11,6 +12,7 @@ from .authenticator import BasicAuthenticator, OrgTokenAuthenticator, ProjectKey
 from .chunk import MemfaultChunk
 from .context import MemfaultCliClickContext
 from .deploy import Deployer
+from .elf import ELFFileInfo
 from .upload import (
     AndroidAppSymbolsUploader,
     BugreportUploader,
@@ -24,7 +26,15 @@ from .upload import (
     XedUploader,
     walk_files,
 )
-from .yocto import find_elf_files_from_image_manifest
+from .yocto import (
+    find_elf_files_from_image,
+    find_elf_files_from_non_native_packages,
+    find_symbol_files_to_upload,
+    tmp_path_from_image,
+)
+
+LOG = logging.getLogger(__name__)
+
 
 pass_memfault_cli_context = click.make_pass_decorator(MemfaultCliClickContext, ensure=True)
 click_argument_path = click.argument("path", type=click.Path(exists=True))
@@ -67,6 +77,9 @@ def cli(ctx: MemfaultCliClickContext, **kwargs):
         httpclient_logger = logging.getLogger("http.client")
         httpclient_logger.setLevel(logging.DEBUG)
 
+        httpclient_logger = logging.getLogger("memfault_cli")
+        httpclient_logger.setLevel(logging.DEBUG)
+
         def httpclient_log(*args) -> None:
             httpclient_logger.log(logging.DEBUG, " ".join(args))
 
@@ -225,20 +238,20 @@ def upload_mar(ctx: MemfaultCliClickContext, path: str, **kwargs):
 
 @cli.command(name="upload-yocto-symbols")
 @click.option(
-    "--manifest",
-    "-m",
+    "--image",
+    "-i",
     required=True,
     help="""
-    The path to the image .manifest as produced by Yocto's do_rootfs (IMAGE_MANIFEST).
-    The .manifest file is expected to reside in the location where Yocto produced the file.
-    The tool will locate the symbol files to upload in the tmp/work/ directory, based on
-    the packages that are listed in the .manifest file.
+    The path to the image .tar/.tar.bz2/.tar.gz/.tar.xz as produced by Yocto's do_image_*.
+    The file is expected to reside in the location where Yocto produced the file
+    (tmp/deploy/images/${PACKAGE_ARCH}/images). The tool will locate the symbol files to
+    upload in the tmp/work/ directory, based on the binaries that are inside the image.
     """,
-    type=click.File(),
+    type=click.Path(exists=True, resolve_path=True, dir_okay=False, path_type=pathlib.Path),
 )
 @click_option_concurrency
 @pass_memfault_cli_context
-def upload_yocto_symbols(ctx: MemfaultCliClickContext, manifest: TextIO, **kwargs):
+def upload_yocto_symbols(ctx: MemfaultCliClickContext, image: pathlib.Path, **kwargs):
     """Upload symbols for a Yocto build.
 
     To see a full example, take a look at the Linux SDK example project:
@@ -250,15 +263,41 @@ def upload_yocto_symbols(ctx: MemfaultCliClickContext, manifest: TextIO, **kwarg
         $ bitbake my-image
         $ memfault --org-token $ORG_TOKEN \\
                    --org acme-inc --project smart-sink \\
-            upload-yocto-symbols --manifest build/tmp/deploy/images/raspberrypi3/my-image-raspberrypi3.rootfs.manifest
+            upload-yocto-symbols --image build/tmp/deploy/images/raspberrypi3/my-image-raspberrypi3.rootfs.tar.bz2
     """
     ctx.obj.update(**kwargs)
 
+    def _image_elfs(image_tar: pathlib.Path) -> Iterable[ELFFileInfo]:
+        with click.progressbar(
+            find_elf_files_from_image(image_tar),
+            label="Finding binaries in image...",
+        ) as bar:
+            yield from bar
+
+    def _tmp_work_elfs(image_tar: pathlib.Path) -> Iterable[ELFFileInfo]:
+        with click.progressbar(
+            find_elf_files_from_non_native_packages(tmp_path_from_image(image_tar)),
+            label="Searching for symbols in tmp/work...",
+        ) as bar:
+            yield from bar
+
+    result = find_symbol_files_to_upload(
+        image_elfs=_image_elfs(image),
+        tmp_work_elfs=_tmp_work_elfs(image),
+    )
+    for missing_elf_info in sorted(result.missing, key=lambda i: i.path):
+        log_func = LOG.debug if missing_elf_info.path.suffix in [".ko"] else LOG.warning
+        log_func(
+            "Could not find symbol file for %s (GNU Build ID: %s)",
+            missing_elf_info.path,
+            missing_elf_info.gnu_build_id,
+        )
+
     ElfSymbolDirectoryUploader(
         ctx=ctx,
-        file_path=manifest.name,  # Not used. TODO: refactor!
+        file_path=image,  # Not used. TODO: refactor!
         authenticator=ctx.create_authenticator(OrgTokenAuthenticator, BasicAuthenticator),
-        file_paths=find_elf_files_from_image_manifest(manifest),
+        file_paths=(elf_info.path for elf_info in result.found),
     ).upload()
 
 
