@@ -1,11 +1,10 @@
 import json
 import logging
-from typing import Any, Dict, List, Optional, Set, Union
+from typing import Any, Dict, List, Optional, Set, Type, Union
 
 import click
 
 from .authenticator import Authenticator
-from .functools_ext import cached_property
 
 LOG = logging.getLogger(__name__)
 
@@ -80,6 +79,11 @@ class XedInfo:  # noqa: B903 - no data classes in Python 3.6
         self.hardware_version = hardware_version
 
 
+class CoreInfo:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(self, device_serial: str):
+        self.device_serial = device_serial
+
+
 class MarInfo:  # noqa: B903 - no data classes in Python 3.6
     def __init__(
         self, device_serial: str, hardware_version: str, software_type: str, software_version: str
@@ -113,8 +117,8 @@ class MemfaultCliClickContext(object):
             raise click.exceptions.UsageError(f"Missing option {self._format_as_option(name)!r}.")
         return value
 
-    def check_required_cli_args(self, *, authenticator: Authenticator):
-        required_args = authenticator.required_args()
+    def check_required_auth_cli_args(self, *, authenticator_type: Type[Authenticator]):
+        required_args = authenticator_type.required_args()
 
         for arg in required_args:
             self._find_obj_or_raise(arg)
@@ -146,9 +150,16 @@ class MemfaultCliClickContext(object):
                 f"Please pass either {' or '.join(sorted(formatted_required_args))}."
             )
 
-    @cached_property
-    def authenticator(self) -> Authenticator:
-        return Authenticator.create_authenticator_given_context_or_raise(self)
+    def create_authenticator(self, *types: Type[Authenticator]) -> Authenticator:
+        assert types
+        for _type in types:
+            try:
+                self.check_required_auth_cli_args(authenticator_type=_type)
+                return _type(self)
+            except click.exceptions.UsageError:
+                pass
+        # Raise for the first missing option of the preferred authenticator type:
+        raise self.check_required_auth_cli_args(authenticator_type=types[0])
 
     @property
     def org(self):
@@ -221,6 +232,15 @@ class MemfaultCliClickContext(object):
     def attributes(self) -> List[Attribute]:
         return [Attribute.from_cli_args(*cli_args) for cli_args in self.obj.get("attribute", [])]
 
+    @property
+    def core_info(self) -> Optional[CoreInfo]:
+        device_serial = self.obj.get("device_serial")
+        if device_serial:
+            return CoreInfo(
+                device_serial=device_serial,
+            )
+        return None
+
     @property
     def xed_info(self) -> XedInfo:
         device_serial = self._find_obj_or_raise("device_serial")
