@@ -2,7 +2,7 @@
 # https://github.com/pydanny/cached-property/blob/master/cached_property.py
 
 
-class cached_property(object):
+class cached_property:  # noqa: N801
     """
     A property that is only computed once per instance and then replaces itself
     with an ordinary attribute. Deleting the attribute resets the property.
